//
//  extension.js
//  SentryBay Screen Capture Protection.
//
//  Created by Paul Saint on 28/09/2022.
//  Copyright © 2023 SentryBay. All rights reserved.
//
//  References:
// 
//  https://gjs.guide/extensions/
//  https://gjs-docs.gnome.org
//  https://stackoverflow.com/questions/tagged/gnome-shell-extensions
//
//  In particular for socket API:
//
//  https://gjs-docs.gnome.org/gio20~2.66p/gio.socket
//

"use strict";

const { GObject, Gio } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const ShellScreenshot = imports.ui.screenshot;

let original_captureScreenshot = ShellScreenshot.captureScreenshot;

const ILOCK_STATUS            = 0x30F11DC2;
const SOCKET_PATH             = '/usr/local/bin/AppProtection/.AppProtection_server';

/**
 * Function to establish whether Screen Capture is enabled.
 */
function screenCaptureEnabled() {
    let enableScreenshot = true;

    let socket = new Gio.Socket({
         "family":Gio.SocketFamily.UNIX,
         "type":Gio.SocketType.STREAM, 
         "protocol":Gio.SocketProtocol.DEFAULT
    });
    socket.init(null);
    let socketAddr = new Gio.UnixSocketAddress({
        "path":SOCKET_PATH
    });
    socket.set_timeout(1);
    try {
        socket.connect(socketAddr, null);

        const msgBufferSize = 24;
        let msgBuffer = new ArrayBuffer(msgBufferSize);
        let bytes = new Uint8Array(msgBuffer);
        let view = new Uint32Array(msgBuffer); // view msgBuffer as an array of 32-bit values
        view[0] = ILOCK_STATUS;
        view[1] = msgBufferSize;

        let bytesSent = socket.send(new Uint8Array(msgBuffer), null);
        if (bytesSent == msgBufferSize)
        {
            let inputStream = new Gio.UnixInputStream({
                fd:socket.get_fd(),
                close_fd:false
            });

            let reply = inputStream.read_bytes(msgBufferSize, null);
            if (reply.get_size() == msgBufferSize)
            {
                for (var i = 0; i < msgBufferSize; i++)
                {
                    bytes[i] = reply.get_data()[i];
                }
                log(`${Me.metadata.name}: response from service=` + view);
                let status = view[2];
                if (status == 1)
                {
                    log(`${Me.metadata.name}: screen capture has been disabled`);
                    enableScreenshot = false;
                }
            }
        }
            
        socket.close();
    }
    catch(error) {
        log(`${Me.metadata.name}: failed to retrieve screen capture protection status from service: error=` + error);
    }

    return enableScreenshot;
}

class Extension {
    /**
     * This function is called when your extension is enabled, which could be
     * done in GNOME Extensions, when you log in or when the screen is unlocked.
     *
     * This is when you should setup any UI for your extension, change existing
     * widgets, connect signals or modify GNOME Shell's behaviour.
     */
    enable() {
        log(`enabling ${Me.metadata.name}`);
        ShellScreenshot.captureScreenshot = async function (texture, geometry, scale, cursor) {
            log(`${Me.metadata.name}: captureScreenshot()`);

            // Suppress Gnome Shell Screenshots by not invoking the Gnome captureScreenshot()
            // function when the extension has determined that screen captures are to be disabled.
            if (screenCaptureEnabled()) await original_captureScreenshot(texture, geometry, scale, cursor);
        }

    }
    
    /**
     * This function is called when your extension is uninstalled, disabled in
     * GNOME Extensions, when you log out or when the screen locks.
     *
     * Anything you created, modifed or setup in enable() MUST be undone here.
     * Not doing so is the most common reason extensions are rejected in review!
     */
    disable() {
        log(`disabling ${Me.metadata.name}`);

        ShellScreenshot.captureScreenshot = original_captureScreenshot;
    }
}

/**
 * This function is called once when your extension is loaded, not enabled. This
 * is a good time to setup translations or anything else you only do once.
 *
 * You MUST NOT make any changes to GNOME Shell, connect any signals or add any
 * MainLoop sources here.
 *
 * @param {ExtensionMeta} meta - An extension meta object, described below.
 * @returns {Object} an object with enable() and disable() methods
 */
function init(meta) {
    log(`initializing ${meta.metadata.name}`);

    return new Extension();
}
